"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCachingMethods = void 0;
const firestore_1 = require("@google-cloud/firestore");
const dataloader_1 = __importDefault(require("dataloader"));
const helpers_1 = require("./helpers");
// https://github.com/graphql/dataloader#batch-function
const orderDocs = (ids) => (docs, keyFn) => {
    const keyFnDef = keyFn !== null && keyFn !== void 0 ? keyFn : ((source) => {
        if (source.id)
            return source.id;
        throw new Error('Could not find ID for object; if using an alternate key, pass in a key function');
    });
    const checkNotUndefined = (input) => {
        return Boolean(input);
    };
    const idMap = docs
        .filter(checkNotUndefined)
        .reduce((prev, cur) => {
        prev[keyFnDef(cur)] = cur;
        return prev;
    }, {});
    return ids.map((id) => idMap[id]);
};
const createCachingMethods = ({ collection, cache, options }) => {
    const loader = new dataloader_1.default(async (ids) => {
        var _a, _b;
        (_a = options === null || options === void 0 ? void 0 : options.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource/DataLoader: loading for IDs: ${ids}`);
        const qSnap = await collection.where(firestore_1.FieldPath.documentId(), 'in', ids).get();
        const documents = qSnap.docs.map(dSnap => dSnap.exists ? dSnap.data() : undefined);
        (_b = options === null || options === void 0 ? void 0 : options.logger) === null || _b === void 0 ? void 0 : _b.debug(`FirestoreDataSource/DataLoader: response count: ${documents.length}`);
        return orderDocs(ids)(documents);
    }, { maxBatchSize: 10 });
    const reviver = (0, helpers_1.reviverFactory)(collection);
    const cachePrefix = `firestore-${collection.path}-`;
    const methods = {
        findOneById: async (id, { ttl } = {}) => {
            var _a;
            (_a = options === null || options === void 0 ? void 0 : options.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource: Running query for ID ${id}`);
            const key = cachePrefix + id;
            const cacheDoc = await cache.get(key);
            if (cacheDoc && ttl) {
                return JSON.parse(cacheDoc, reviver);
            }
            const doc = await loader.load(id);
            if (Number.isInteger(ttl)) {
                await cache.set(key, JSON.stringify(doc, helpers_1.replacer), { ttl });
            }
            return doc;
        },
        findManyByIds: async (ids, args = {}) => {
            var _a;
            (_a = options === null || options === void 0 ? void 0 : options.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource: Running query for IDs ${ids}`);
            return await Promise.all(ids.map(async (id) => await methods.findOneById(id, args)));
        },
        deleteFromCacheById: async (id) => {
            var _a;
            (_a = options === null || options === void 0 ? void 0 : options.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource: Deleting from cache ${id}`);
            loader.clear(id);
            await cache.delete(cachePrefix + id);
        },
        /**
         * Loads an item or items into DataLoader and optionally the cache (if TTL is specified)
         * Use this when running a query outside of the findOneById/findManyByIds methos
         * that automatically and transparently do this
         */
        primeLoader: async (docs, ttl) => {
            docs = Array.isArray(docs) ? docs : [docs];
            for (const doc of docs) {
                loader.prime(doc.id, doc);
                const key = cachePrefix + doc.id;
                if (!!ttl || !!(await cache.get(key))) {
                    await cache.set(key, JSON.stringify(doc, helpers_1.replacer), { ttl });
                }
            }
        },
        dataLoader: loader,
        cache,
        cachePrefix,
        reviver,
        replacer: helpers_1.replacer
    };
    return methods;
};
exports.createCachingMethods = createCachingMethods;
//# sourceMappingURL=cache.js.map