"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirestoreDataSource = void 0;
const utils_keyvaluecache_1 = require("@apollo/utils.keyvaluecache");
const helpers_1 = require("./helpers");
const cache_1 = require("./cache");
const placeholderHandler = () => {
    throw new Error('DataSource not initialized');
};
class FirestoreDataSource {
    /**
     *
     * @param query
     * @param options
     */
    async findManyByQuery(queryFunction, { ttl } = {}) {
        var _a;
        const qSnap = await queryFunction(this.collection).get();
        const results = qSnap.docs.map(dSnap => dSnap.data());
        // prime these into the dataloader and maybe the cache
        if (this.dataLoader && results) {
            await this.primeLoader(results, ttl);
        }
        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource/findManyByQuery: complete. rows: ${qSnap.size}, Read Time: ${qSnap.readTime.toDate()}`);
        return results;
    }
    async createOne(newDoc, { ttl } = {}) {
        var _a, _b;
        if ('id' in newDoc) {
            return await this.updateOne(newDoc);
        }
        else {
            const dRef = await this.collection.add(newDoc);
            const dSnap = await dRef.get();
            const result = dSnap.data();
            if (result) {
                await this.primeLoader(result, ttl);
            }
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource/createOne: created id: ${(_b = result === null || result === void 0 ? void 0 : result.id) !== null && _b !== void 0 ? _b : ''}`);
            return result;
        }
    }
    async deleteOne(id) {
        var _a;
        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource/deleteOne: deleting id: '${id}'`);
        const response = await this.collection.doc(id).delete();
        await this.deleteFromCacheById(id);
        return response;
    }
    async updateOne(data) {
        var _a;
        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource/updateOne: Updating doc id ${data.id}`);
        await this.collection
            .doc(data.id)
            .set(data);
        const dSnap = await this.collection.doc(data.id).get();
        const result = dSnap.data();
        if (result) {
            await this.primeLoader(result);
        }
        return result;
    }
    async updateOnePartial(id, data) {
        var _a;
        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug(`FirestoreDataSource/updateOnePartial: Updating doc id ${id}`);
        await this.collection
            .doc(id)
            .set(data, { merge: true });
        const dSnap = await this.collection.doc(id).get();
        const result = dSnap.data();
        if (result) {
            await this.primeLoader(result);
        }
        return result;
    }
    constructor(collection, options) {
        var _a, _b;
        // these get set by the initializer but they must be defined or nullable after the constructor
        // runs, so we guard against using them before init
        this.findOneById = placeholderHandler;
        this.findManyByIds = placeholderHandler;
        this.deleteFromCacheById = placeholderHandler;
        this.primeLoader = placeholderHandler;
        this.reviver = placeholderHandler;
        this.replacer = placeholderHandler;
        (_a = options === null || options === void 0 ? void 0 : options.logger) === null || _a === void 0 ? void 0 : _a.debug('FirestoreDataSource started');
        if (!(0, helpers_1.isFirestoreCollection)(collection)) {
            throw new Error('FirestoreDataSource must be created with a Firestore collection (from @google-cloud/firestore)');
        }
        this.cache = (_b = options === null || options === void 0 ? void 0 : options.cache) !== null && _b !== void 0 ? _b : new utils_keyvaluecache_1.InMemoryLRUCache();
        this.logger = options === null || options === void 0 ? void 0 : options.logger;
        this.collection = collection.withConverter((0, helpers_1.FirestoreConverter)());
        const methods = (0, cache_1.createCachingMethods)({
            collection: this.collection,
            cache: this.cache,
            options
        });
        Object.assign(this, methods);
    }
}
exports.FirestoreDataSource = FirestoreDataSource;
//# sourceMappingURL=datasource.js.map